<?php
/**
 * 2024 Addify
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    Addify
 *  @copyright 2024 Addify
 *  @license   http://opensource.org/licenses/afl-3.0.php
 */

if (!defined('_PS_VERSION_')) {
    exit;
}
include_once(_PS_MODULE_DIR_).'addifyextendedhideprice/classes/ExtHidePrice.php';
class Addifyextendedhideprice extends Module
{
    protected $config_form = false;

    public function __construct()
    {
        $this->name = 'addifyextendedhideprice';
        $this->tab = 'front_office_features';
        $this->version = '1.4.0';
        $this->author = 'Addify';
        $this->need_instance = 0;
        $this->module_key = 'c9989c352e3e67fe371bb6f5d1763817';

        /**
         * Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6)
         */
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('Addify Hide Price Pro');
        $this->description = $this->l('Hide product prices and Add to Cart button based on customer groups, products, or categories with customizable display options.');

        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_);
    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */
    public function install()
    {
        Configuration::updateValue('addify_AS_live_mode', false);

        include(dirname(__FILE__).'/sql/install.php');
        return parent::install() &&
            $this->registerHook('displayHeader') && $this->registerHook('displayProductPriceBlock') && $this->installTab();
    }

    public function uninstall()
    {
        Configuration::deleteByName('addify_AS_live_mode');
        
        include(dirname(__FILE__).'/sql/uninstall.php');
        return parent::uninstall() && $this->unregisterHook('displayHeader') && $this->unregisterHook('displayProductPriceBlock')  && $this->uninstallTab();
    }
    public function installTab()
    {
        $tab = new Tab();
        $tab->active = 1;
        if (true === Tools::version_compare(_PS_VERSION_, '1.7.0.0', '>=')) {
            $tab->id_parent = 0;
        } else {
            $tab->id_parent = 0;
        }
        $tab->class_name = 'AdminParentHidePrice';
        $tab->name = array();
        foreach (Language::getLanguages(true) as $lang) {
            $tab->name[$lang['id_lang']] = $this->l('Extended Hide Price');
        }
        $tab->add();

        $tab = new Tab();
        $tab->active = 1;
        $tab->id_parent = (int)Tab::getIdFromClassName('AdminParentHidePrice');
        ;
        $tab->class_name = 'AdminHidePrice';
        $tab->name = array();
        foreach (Language::getLanguages(true) as $lang) {
            $tab->name[$lang['id_lang']] = $this->l('Manage Extended Hide Price');
        }
        $tab->module = $this->name;
        return $tab->add();
    }

    public function uninstallTab()
    {
        $id_tab = (int)Tab::getIdFromClassName('AdminHidePrice');
        if ($id_tab) {
            $tab = new Tab($id_tab);
            $tab->delete();
        }
        $id_tab = (int)Tab::getIdFromClassName('AdminParentHidePrice');
        if ($id_tab) {
            $tab = new Tab($id_tab);
            return $tab->delete();
        }
        return false;
    }

    public function getContent()
    {
        if (!$this->active) {
            $this->context->controller->errors[] = $this->l('Access Denied...Module is disabled.');
        } else {

            if (Tools::isSubmit('addifyhidePriceConfigBtn'.$this->name)) {
                $this->addifyAddUpdateConfigFormValues();
            }

            $this->postProcess();

            $active_tab = Tools::getValue('active_tab');

            if (!$active_tab) {
               $active_tab = 1;
            }

            $this->context->smarty->assign(array(
                'active_tab' => $active_tab,
            ));

            if (Tools::isSubmit('add'.$this->name) || Tools::isSubmit('update'.$this->name)) {
                $this->context->smarty->assign(array(
                    'configurationDisplayForm' => $this->configurationDisplayForm(),
                    'renderForm' => $this->renderForm(),    
                    'renderList' => $this->renderList(),
                ));

                return $this->display(__FILE__,'views/templates/admin/rules.tpl');
            }

            $this->context->smarty->assign(array(
                'configurationDisplayForm' => $this->configurationDisplayForm(),
                'renderList' => $this->renderList(),
                'renderForm' => $this->renderForm(),
            ));

            return $this->display(__FILE__,'views/templates/admin/configuration.tpl');
        }
    }

    protected function postProcess()
    {
        if (Tools::isSubmit('submitAddifyextendedhideprice')) {
            $this->processSave();
        } elseif (Tools::isSubmit('status'.$this->name)) {
            if (!ExtHidePrice::setStatus('active', (int)Tools::getValue('id_addifyextendedhideprice'))) {
                $this->context->controller->errors[] = $this->l('Status update failed.');
            } else {
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=6&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name.'&active_tab=2');
            }
        } elseif (Tools::isSubmit('delete'.$this->name)) {
            if (!Validate::isLoadedObject($hideprice = new ExtHidePrice((int)Tools::getValue('id_addifyextendedhideprice')))) {
                $this->context->controller->errors[] = $this->l('Object not found.');
            } else {
                if (!$hideprice->delete()) {
                    $this->context->controller->errors[] = $this->l('Rule cannot be deleted.');
                } else {
                    $this->context->controller->confirmations[] = $this->l('Rule deleted successfully.');
                }
            }
        } elseif (Tools::isSubmit('submitBulkdelete'.$this->name)) {
            $addifyextendedhidepriceBox = Tools::getValue('addifyextendedhidepriceBox');
            if (isset($addifyextendedhidepriceBox) && $addifyextendedhidepriceBox) {
                $deleted = 0;
                foreach ($addifyextendedhidepriceBox as $id_addifyextendedhideprice) {
                    if (!Validate::isLoadedObject($hideprice = new ExtHidePrice((int)$id_addifyextendedhideprice))) {
                        $this->context->controller->errors[] = $this->l('Object not found.');
                        break;
                    } else {
                        if ($hideprice->delete()) {
                            $deleted++;
                        }
                    }
                }
                if (count($this->context->controller->errors)) {
                    $this->context->controller->errors;
                } else {
                    $this->context->controller->confirmations[] = sprintf($this->l('%s records deleted successfully.'), $deleted);
                }
            }
        }
    }

    public function configurationDisplayForm()
    {
        $configurationDisplayFormVar = array(
        'form' => array(

            'legend' =>  array(
                'title' => $this->l('General Settings'),
                'icon' => 'icon-cogs'
            ),

            'input' => array(

                array( 
                'type' => 'switch',
                'label' => $this->l('Module'),
                'name' => 'addify_AS_live_mode',
                'is_bool' => true,
                'desc' => $this->l('Enable disable module from here.'),
                    'values' => array(
                        array(
                            'id' => 'active_on',
                            'value' => 1,
                            'label' => $this->l('Enabled')
                        ),
                        array(
                            'id' => 'active_off',
                            'value' => 0,
                            'label' => $this->l('Disabled')
                        )
                    ),
                ),
            ),

            'submit' => array(
                'title' => $this->l('Save'),
                'class' => 'btn btn-default pull-right'
            )
        ),
    );

    $helper = new HelperForm(); // generate html elements
    $helper->show_toolbar = false;
    $helper->table = $this->table;
    $helper->default_form_language = $this->context->language->id;
    $helper->identifier = $this->identifier;
    $helper->submit_action = 'addifyhidePriceConfigBtn'.$this->name;

    $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
    $helper->token = Tools::getAdminTokenLite('AdminModules');

    $helper->tpl_vars = array(
        'fields_value' => $this->addifyConfigFormValues(),
        'languages' => $this->context->controller->getLanguages(),
        'id_language' => $this->context->language->id,
    );
        return $helper->generateForm(array($configurationDisplayFormVar));

    }

    /**
     * Set values for the inputs.
    */
    public function addifyConfigFormValues()
    {
        $configFromValue = array();
        if(Tools::isSubmit('addifyhidePriceConfigBtn'.$this->name) === false) {
          $configFromValue['addify_AS_live_mode'] = Configuration::get('addify_AS_live_mode');
        }

        else { 
            $configFromValue['addify_AS_live_mode'] = Tools::getValue('addify_AS_live_mode');
        }

      return $configFromValue;
    }

    /**
     * Save configuration form data
    */
    public function addifyAddUpdateConfigFormValues()
    {
        $addifyConfigAddUpdate = $this->addifyConfigFormValues();
        Configuration::updateValue('addify_AS_live_mode',$addifyConfigAddUpdate['addify_AS_live_mode']);
       
        Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=4&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name); 
    }

    protected function renderList()
    {
        $fields_list = array(
            'id_addifyextendedhideprice' => array(
                'title' => $this->l('ID'),
                'width' => 120,
                'type' => 'text',
                'search' => false,
                'orderby' => false
            ),
            'title' => array(
                'title' => $this->l('Title'),
                'width' => 140,
                'type' => 'text',
                'search' => false,
                'orderby' => false
            ),
            'priority' => array(
                'title' => $this->l('Priority'),
                'width' => 140,
                'type' => 'text',
                'search' => false,
                'orderby' => false
            ),
            'active' => array(
                'title' => $this->l('Status'),
                'width' => 140,
                'active' => 'status',
                'type' => 'bool',
                'search' => false,
                'orderby' => false
            ),
            'date_add' => array(
                'title' => $this->l('Added Date'),
                'width' => 140,
                'type' => 'datetime',
                'search' => false,
                'orderby' => false
            ),
            'date_upd' => array(
                'title' => $this->l('Update Date'),
                'width' => 140,
                'type' => 'datetime',
                'search' => false,
                'orderby' => false
            ),
        );

        // multishop feature
        if (Shop::isFeatureActive()) {
            $this->$fields_list['id_shop'] = array(
                'title' => $this->l('ID Shop'),
                'align' => 'center',
                'width' => 25,
                'type' => 'int'
            );
        }

        $helper = new HelperList();
        $helper->shopLinkType = '';
        $helper->simple_header = false;
        $helper->identifier = 'id_addifyextendedhideprice';
        $helper->actions = array('edit', 'delete');
        $helper->bulk_actions = array(
            'delete' => array(
                'text' => $this->l('Delete selected'),
                'icon' => 'icon-trash',
                'confirm' => $this->l('Delete selected items?')
            )
        );
        $helper->listTotal = count(ExtHidePrice::getListContent());
        $helper->show_toolbar = true;
        $helper->toolbar_btn['new'] =  array(
            'href' => AdminController::$currentIndex.'&configure='.$this->name.'&add'.$this->name.'&general_settings_here'.$this->name.'&token='.Tools::getAdminTokenLite('AdminModules').'&active_tab=2',
            'desc' => $this->l('Add new')
        );

        $helper->title = $this->displayName;
        $helper->table = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name.'&active_tab=2';
        return $helper->generateList(ExtHidePrice::getListContent(), $fields_list);
    }

    /**
     * Create the form that will be displayed in the configuration of your module.
     */
    protected function renderForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->show_cancel_button = false;
        $helper->table = 'addifyextendedhideprice';
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = 'id_'.$this->table;
        $helper->submit_action = 'submitAddifyextendedhideprice';
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name.'&active_tab=2';
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->back_url = $helper->currentIndex.'&token='.$helper->token;
        $products = array();

        if (Tools::getValue('id_addifyextendedhideprice')) {
            $hidepriceBlock = new ExtHidePrice((int)Tools::getValue('id_addifyextendedhideprice'));
            $post_accessories_tab = explode(',', $hidepriceBlock->product);
            foreach ($post_accessories_tab as $id_product) {
                if (!$this->haveThisProduct($id_product, $products) &&
                    $product = ExtHidePrice::getProductById($id_product)) {
                    $products[] = $product;
                }
            }
        } else {
            $products = array();
        }
        if ($post_accessories = Tools::getValue('inputRestrictedProducts')) {
            $post_accessories_tab = explode('-', $post_accessories);
            foreach ($post_accessories_tab as $id_product) {
                if (!$this->haveThisProduct($id_product, $products) &&
                    $product = ExtHidePrice::getProductById($id_product)) {
                    $products[] = $product;
                }
            }
        }

        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
            'products' => $products,
            'context_link' => Context::getContext()->link,
        );

        return $helper->generateForm(array($this->getConfigForm()));
    }
    public function haveThisProduct($id_product, $products)
    {
        foreach ($products as $product) {
            if ((int)$product['id_product'] == (int)$id_product) {
                return true;
            }
        }
        return false;
    }

    /**
     * Create the structure of your form.
     */
    protected function getConfigForm()
    {
        $access_type = array(
            array('id' => '1', 'name' => $this->l('All Product')),
            array('id' => '2', 'name' => $this->l('Selected Product')),
        );
        
        // Categories
        if (Tools::getValue('id_addifyextendedhideprice')) {
            $hidepriceBlock = new ExtHidePrice((int)Tools::getValue('id_addifyextendedhideprice'));
            $selected_categories = explode(',', $hidepriceBlock->categories);
        } else {
            $selected_categories = array();
        }

        // Getting Suppliers
        $Suppliers_array = Supplier::getSuppliers(false, $this->context->language->id, false, false, false, false);
        $allSuppliers = array();

        foreach ($Suppliers_array as $Supplier) {
            $allSuppliers[] = array('name' => $Supplier['name'], 'id' => $Supplier['id_supplier']);
        }

        // Getting Manufacturers
        $Manufacturers_array = Manufacturer::getManufacturers(false,$this->context->language->id, false, false, false);
        $allManufacturers = array();

        foreach ($Manufacturers_array as $Manufacturer) {
            $allManufacturers[] = array('name' => $Manufacturer['name'], 'id' => $Manufacturer['id_manufacturer']);
        }

        $fields_form = array(

            'form' => array(

                'legend' => array(
                    'title' => $this->l('Rules'),
                    'icon' => 'icon-cogs',
                ),

                'input' => array(

                    array(
                        'type' => 'hidden',
                        'name' => 'id_addifyextendedhideprice'
                    ),

                    array(
                        'type'  => 'text',
                        'label' => $this->l('Rule Name'),
                        'name'  => 'title',
                        'col' => 4,
                    ),

                    array(
                        'type' => 'text',
                        'label' => $this->l('Rule Priority'),
                        'name' => 'priority',
                        'col' => 4
                    ),

                    array(
                        'type' => 'switch',
                        'label' => $this->l('Active Rule'),
                        'name' => 'active',
                        'is_bool' => true,
                        'desc' => $this->l('Enable/Disable this hide price rule.'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),

                    array(
                        'type' => 'switch',
                        'label' => $this->l('Hide Price'),
                        'name' => 'hideprice',
                        'desc' => $this->l('Enable/disable hide price. If enabled, the product prices will be hidden.'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),

                    array(
                        'type' => 'switch',
                        'label' => $this->l('Custom Message'),
                        'desc' => $this->l('If enabled, this message will be shown on selected products where prices are hidden.'),
                        'name' => 'enabled_disabled_custom_message_mode',
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),

                    array(
                        'type' => 'textarea',
                        'label' => $this->l('Add Custom Message'),
                        'name' => 'message',
                        'autoload_rte' => true,
                        'id'=> "message",
                        'lang' => true,
                        'col' => 8,
                        'hint' => $this->l('Invalid characters:').' <>;=#{}'
                    ),

                    // ====================================================================
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Hide Add to Cart Button'),
                        'name' => 'hidecart',
                        'desc' => $this->l('Enable/disable the "Add to Cart" button. If enabled, the button will be hidden.'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),

                    array(
                        'type' => 'switch',
                        'label' => $this->l('Custom Button'),
                        'name' => 'active_custom_btn',
                        'desc' => $this->l('Enable/disable the custom button. If enabled, it will replace the "Add to Cart" button.'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),

                    array(
                        'type' => 'text',
                        'label' => $this->l('Custom Button Title'),
                        'name' => 'custom_btn_title',
                        'desc' => $this->l('Enter a custom button title that will be displayed on the button.'),
                        'col' => 4
                    ),

                    array(
                        'type' => 'text',
                        'label' => $this->l('Custom button Link Field'),
                        'name' => 'custom_btn_link',
                        'desc' => $this->l('Enter the link for the custom button. When the user clicks it, they will be redirected to the provided link.'),
                        'col' => 4
                    ),

                    array(
                        'type' => 'color',
                        'label' => $this->l('Custom Button Text Color'),
                        'name' => 'custom_btn_text_color',
                        'col' => 4
                    ),

                    array(
                        'type' => 'color',
                        'label' => $this->l('Custom Button Background Color'),
                        'name' => 'custom_btn_background_color',
                        'col' => 4
                    ),
                    // ======================================================================
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Customer Group'),
                        'name' => 'groups_activate',
                        'id' => 'groups_activate',
                        'desc' => $this->l('Enable/disable customer group.'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),

                    array(
                        'type' => 'group',
                        'label' => $this->l('Group'),
                        'name' => 'groupBox',
                        'values' => Group::getGroups(Context::getContext()->language->id),
                        'info_introduction' => $this->l('You now have three default customer groups.'),
                        'desc' => $this->l('Mark any/all of the customer group(s) which you would like to apply the rule.'),
                    ),

                    array(
                        'type' => 'switch',
                        'label' => $this->l('All Products'),
                        'name' => 'type',
                        'id' => 'type',
                        'desc' => $this->l('Enable/disable all products.'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),

                    array(
                        'type' => 'switch',
                        'label' => $this->l('Product Selection'),
                        'name' => 'selected_products_activate',
                        'id' => 'selected_products_activate',
                        'desc' => $this->l('Enable/disable search products.'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),

                    array(
                        'type' => 'hideprice_product',
                        'label' => $this->l('Search product'),
                        'name' => 'product',
                        'col' => 6,
                        'prefix' => '<i class="icon-search"></i>',
                        'hint' => $this->l('Product(s) Price will be show for selected product.'),
                    ),

                    array(
                        'type' => 'switch',
                        'label' => $this->l('Category Selection'),
                        'name' => 'category_activate',
                        'id' => 'category_activate',
                        'desc' => $this->l('Enable/disable categories.'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),

                    array(
                        'type'  => 'categories',
                        'label' => $this->l('Categories'),
                        'name'  => 'category',
                        'col' => 6,
                        'tree'  => array(
                            'id' => 'type_category',
                            'use_checkbox' => true,
                            'disabled_categories' => null,
                            'selected_categories' => $selected_categories,
                            'root_category' => Context::getContext()->shop->getCategory()
                        )
                    ),

                    array(
                        'type' => 'switch',
                        'label' => $this->l('Supplier Selection'),
                        'name' => 'supplier_activate',
                        'id' => 'supplier_activate',
                        'desc' => $this->l('Enable/disable supplier.'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),

                    array(
                        'type' => 'select',
                        'label' => $this->l('Supplier'),
                        'name' => 'Supplier',
                        'id' => 'Supplier',
                        'multiple' => true,
                        'desc' => $this->l('Choose the supplier for whom you want to apply for hide price and add to cart button.'),
                        'options' => array(
                            'query' => $allSuppliers,
                            'id' => 'id',
                            'name' => 'name',
                        ),
                    ),

                    array(
                        'type' => 'switch',
                        'label' => $this->l('Manufacturer Selection'),
                        'name' => 'manufacturer_activate',
                        'id' => 'manufacturer_activate',
                        'desc' => $this->l('Enable/disable manufacturer.'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),

                    array(
                        'type' => 'select',
                        'label' => $this->l('Manufacturer'),
                        'name' => 'Manufacturer',
                        'id' => 'Manufacturer',
                        'multiple' => true,
                        'desc' => $this->l('Choose the manufacturer for whom you want to apply for hide price and add to cart button.'),
                        'options' => array(
                            'query' => $allManufacturers,
                            'id' => 'id',
                            'name' => 'name',
                        ),
                    ),

                    array(
                        'type' => 'switch',
                        'label' => $this->l('Price Based Selection'),
                        'name' => 'price_based_activate',
                        'id' => 'price_based_activate',
                        'desc' => $this->l('If active, products with prices between the min and max values will be hidden.'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),

                    array(
                        'type' => 'text',
                        'label' => $this->l('Minumum Price'),
                        'desc' => $this->l('Enter minmum price'),
                        'name' => 'min_price',
                        'id' => 'min_price',
                        'col' => 2,
                    ),

                    array(
                        'type' => 'text',
                        'label' => $this->l('Maximum Price'),
                        'desc' => $this->l('Enter maximum price'),
                        'name' => 'max_price',
                        'id' => 'max_price',
                        'col' => 2,
                    ),

                    array(
                        'type' => 'switch',
                        'label' => $this->l('Stock Based Selection'),
                        'name' => 'stock_based_activate',
                        'id' => 'stock_based_activate',
                        'desc' => $this->l('Enable/disable stock based.'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                   
                    array(
                        'type' => 'switch',
                        'col' => 8,
                        'label' => $this->l(' Product With Zero Stock'),
                        'name' => 'zero_stock_activate',
                        'id' => 'zero_stock_activate',
                        'desc' => $this->l('If enable so the rule will be applied on out of stock products.'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('Disabled')
                            )
                        )
                    ),

                    array(
                        'type' => 'switch',
                        'col' => 8,
                        'label' => $this->l('Custom Stock Value'),
                        'name' => 'custom_stock_activate',
                        'id' => 'custom_stock_activate',
                        'desc' => $this->l('Enable/disable custom stock value.'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('Disabled')
                            )
                        )
                    ),

                    array(
                        'type' => 'text',
                        'col' => 4,
                        'label' => $this->l('Custom Stock Value'),
                        'name' => 'custom_stock_value',
                        'id' => 'custom_stock_value',
                        'desc' => $this->l('Enter the stock value. Products with stock lower than this value will have their price hidden.'),
                    ),
                    
                    array(
                        'type' => 'switch',
                        'label' => $this->l(' Rule Scheduling'),
                        'desc' => $this->l('If activate, the rule will be applied within the specified start and end dates.'),
                        'name' => 'rule_date_activate',
                        'id' => 'rule_date_activate',
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),

                    array(
                        'type' => 'datetime',
                        'label' => $this->l('Rule Start Date'),
                        'name' => 'module_rule_start_date',
                        'col' => 8
                    ),

                    array(
                        'type' => 'datetime',
                        'label' => $this->l('Rule Expire Date'),
                        'name' => 'module_rule_expire_date',
                        'col' => 8
                    ),

                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
                'buttons' => array(
                    'cancel' => array(
                        'title' => $this->l('Cancel'),
                        'class' => 'btn btn-default',
                        'icon' => 'process-icon-cancel',
                        'href' => AdminController::$currentIndex.'&configure='.$this->name.'&token='.Tools::getAdminTokenLite('AdminModules').'&active_tab=2',
                    ),
                )
            ),
        );

        if (Shop::isFeatureActive()) {
            $this->$fields_form['input'][] = array(
                'type' => 'shop',
                'label' => $this->l('Shop association'),
                'name' => 'checkBoxShopAsso',
            );
        }
        return $fields_form;
    }
    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        $fields_value = array();
        $accessed_groups_ids = array();
        $fields_value = array(
            'message' => Tools::getValue('message'),
        );

        if (Tools::getValue('id_addifyextendedhideprice') && (Tools::isSubmit('submitAddifyextendedhideprice') === false)) {
            $hidepriceBlock = new ExtHidePrice((int)Tools::getValue('id_addifyextendedhideprice'));
            $fields_value['id_addifyextendedhideprice'] = $hidepriceBlock->id;
            $fields_value['title'] = $hidepriceBlock->title;
            $fields_value['priority'] = $hidepriceBlock->priority;
            $fields_value['active'] = $hidepriceBlock->active;
            $fields_value['hideprice'] = $hidepriceBlock->hideprice;

            $fields_value['hidecart'] = $hidepriceBlock->hidecart;
            $fields_value['active_custom_btn'] = $hidepriceBlock->active_custom_btn;
            $fields_value['custom_btn_title'] = $hidepriceBlock->custom_btn_title;
            $fields_value['custom_btn_link'] = $hidepriceBlock->custom_btn_link;
            $fields_value['custom_btn_text_color'] = $hidepriceBlock->custom_btn_text_color;
            $fields_value['custom_btn_background_color'] = $hidepriceBlock->custom_btn_background_color;

            $fields_value['rule_date_activate'] = $hidepriceBlock->rule_date_activate;
            $fields_value['module_rule_start_date'] = $hidepriceBlock->module_rule_start_date;
            $fields_value['module_rule_expire_date'] = $hidepriceBlock->module_rule_expire_date;

            $fields_value['groups_activate'] = $hidepriceBlock->groups_activate;
            $fields_value['groupBox'] = explode(',', $hidepriceBlock->groups);

            $fields_value['type'] = $hidepriceBlock->product_type;
            $fields_value['selected_products_activate'] = $hidepriceBlock->selected_products_activate;
            $fields_value['product'] = explode(',', $hidepriceBlock->product);
            $fields_value['category_activate'] = $hidepriceBlock->category_activate;
            $fields_value['category'] = explode(',', $hidepriceBlock->categories);
            $fields_value['supplier_activate'] = $hidepriceBlock->supplier_activate;

            if (!$hidepriceBlock->Supplier) {
                $fields_value['Supplier[]'] = '';
            } else {
                $fields_value['Supplier[]'] = explode(',', $hidepriceBlock->Supplier);
            }

            $fields_value['manufacturer_activate'] = $hidepriceBlock->manufacturer_activate;

            if (!$hidepriceBlock->Manufacturer) {
                $fields_value['Manufacturer[]'] = '';
            } else {
                $fields_value['Manufacturer[]'] = explode(',', $hidepriceBlock->Manufacturer);
            }

            $fields_value['price_based_activate'] = $hidepriceBlock->price_based_activate;
            $fields_value['min_price'] = $hidepriceBlock->min_price;
            $fields_value['max_price'] = $hidepriceBlock->max_price;
            $fields_value['stock_based_activate'] = $hidepriceBlock->stock_based_activate;
            $fields_value['zero_stock_activate'] = $hidepriceBlock->zero_stock_activate;
            $fields_value['custom_stock_activate'] = $hidepriceBlock->custom_stock_activate;
            $fields_value['custom_stock_value'] = $hidepriceBlock->custom_stock_value;

            $fields_value['enabled_disabled_custom_message_mode'] = $hidepriceBlock->enabled_disabled_custom_message_mode;
            $fields_value['message'] = $hidepriceBlock->displaytext;
            $accessed_groups_ids = $fields_value['groupBox'];
        } 
        else 
        {
            $fields_value['id_addifyextendedhideprice'] = Tools::getValue('id_addifyextendedhideprice');
            $fields_value['active'] = (int)Tools::getValue('active');
            $fields_value['priority'] = Tools::getValue('priority');
            $fields_value['title'] = Tools::getValue('title');
            $fields_value['hideprice'] = (int)Tools::getValue('hideprice');

            $fields_value['hidecart'] = (int)Tools::getValue('hidecart');
            $fields_value['active_custom_btn'] = Tools::getValue('active_custom_btn');
            $fields_value['custom_btn_title'] = Tools::getValue('custom_btn_title');
            $fields_value['custom_btn_link'] = Tools::getValue('custom_btn_link');
            $fields_value['custom_btn_text_color'] = Tools::getValue('custom_btn_text_color');
            $fields_value['custom_btn_background_color'] = Tools::getValue('custom_btn_background_color');

            $fields_value['rule_date_activate'] = (int)Tools::getValue('rule_date_activate');
            $fields_value['module_rule_start_date'] = Tools::getValue('module_rule_start_date');
            $fields_value['module_rule_expire_date'] = Tools::getValue('module_rule_expire_date');

            $fields_value['groups_activate'] = (int)Tools::getValue('groups_activate');
            if (!Tools::getValue('groupBox')) {
                $fields_value['groupBox'] =  Tools::getValue('groupBox');
            } else {
                $fields_value['groupBox'] = implode(',', Tools::getValue('groupBox'));
            }

            $productsActivateValue = Tools::getValue('type', 1); 
            if ((int)$productsActivateValue === 1) {
                $fields_value['type'] = 1;
            } else {
                $fields_value['type'] = (int)$productsActivateValue;
            }

            $fields_value['selected_products_activate'] = (int)Tools::getValue('selected_products_activate');
            if (!Tools::getValue('product')) {
                $fields_value['product'] = Tools::getValue('product');
            } else {
                $fields_value['product'] = implode(',', Tools::getValue('product'));
            }

            $fields_value['category_activate'] = (int)Tools::getValue('category_activate');
            if (Tools::getValue('category')) {
                $fields_value['category'] = implode(',', Tools::getValue('category'));
            } else {
                $fields_value['category'] = '' ;
            }

            $fields_value['supplier_activate'] =  (int)Tools::getValue('supplier_activate');
            $supplierValues = Tools::getValue('Supplier');
            if ($supplierValues) {
                $fields_value['Supplier[]'] = implode(',', $supplierValues);
            } else {
                $fields_value['Supplier[]'] = '' ;
            }

            $fields_value['manufacturer_activate'] = (int)Tools::getValue('manufacturer_activate');
            $manufacturerValues = Tools::getValue('Manufacturer');
            if ($manufacturerValues) {
                $fields_value['Manufacturer[]'] = implode(',', $manufacturerValues);
            }
            else {
                $fields_value['Manufacturer[]'] = '' ;
            }

            $fields_value['price_based_activate'] = (int)Tools::getValue('price_based_activate');
            $fields_value['min_price'] = Tools::getValue('min_price');
            $fields_value['max_price'] = Tools::getValue('max_price');

            $fields_value['stock_based_activate'] = (int)Tools::getValue('stock_based_activate');
            $fields_value['zero_stock_activate'] = Tools::getValue('zero_stock_activate');
            $fields_value['custom_stock_activate'] = Tools::getValue('custom_stock_activate');
            $fields_value['custom_stock_value'] = Tools::getValue('custom_stock_value');

            $fields_value['enabled_disabled_custom_message_mode'] = (int)Tools::getValue('enabled_disabled_custom_message_mode');

            /* getting multilingual fields*/
            $languages = Language::getLanguages(false);
            $fields_value['message'] = []; // Initialize as an empty array
            foreach ($languages as $lang) {
                $message = Tools::getValue('message_'.$lang['id_lang']);

                // getting message
                if (!empty($message) && !Validate::isCleanHtml($message)) {
                    $this->context->controller->errors[] = sprintf(
                        $this->l('Invalid message content in %s.'),
                        Language::getIsoById($lang['id_lang'])
                    );
                } else {
                    $fields_value['message'][(int)$lang['id_lang']] = $message;
                }
            }
            $accessed_groups_ids =  Tools::getValue('groupBox');
        }

        if (!is_array($accessed_groups_ids)) {
            $accessed_groups_ids = array();
        }
        
        // Added values of object Group
        $groups = Group::getGroups($this->context->language->id);
        if (empty($accessed_groups_ids)) {
            $preselected = array(Configuration::get('PS_UNIDENTIFIED_GROUP'), Configuration::get('PS_GUEST_GROUP'), Configuration::get('PS_CUSTOMER_GROUP'));
            $accessed_groups_ids = array_merge($accessed_groups_ids, $preselected);
        }
        foreach ($groups as $group) {
            $fields_value['groupBox_'.$group['id_group']] = (isset($accessed_groups_ids) && $accessed_groups_ids)? Tools::getValue('groupBox_'.$group['id_group'], (in_array($group['id_group'], $accessed_groups_ids))) : true;
        }

        return $fields_value;
    }
    /**
     * Save form data.
     */
    protected function processSave()
    {
        $call_back = 'add';
        $form_values = $this->getConfigFormValues();

        if (!isset($form_values) && !$form_values) {
            return $this->context->controller->errors[] = $this->l('Empty post values.');
        }
        if (!is_numeric($form_values['priority'])) {
            $this->context->controller->errors[] = $this->l('Priority value is not valid .');
        }

        $start_date = $form_values['module_rule_start_date'];
        $expire_date = $form_values['module_rule_expire_date'];

        if ($start_date == '0000-00-00 00:00:00') {
            $start_date = '';
        }
        if ($expire_date == '0000-00-00 00:00:00') {
            $expire_date = '';
        }

        if ((!empty($start_date) && $start_date != '0000-00-00 00:00:00') || (!empty($expire_date) && $expire_date != '0000-00-00 00:00:00')) {
            if (empty($start_date) || empty($expire_date)) {
                $this->context->controller->errors[] = $this->l('Rule schedule start and expiry dates must be filled.');
            }
            if (!empty($start_date) && !empty($expire_date) && $start_date > $expire_date) {
                $this->context->controller->errors[] = $this->l('The rule expiry date must be after the rule start date.');
            }
        }

        // Save the cleaned-up values back to $form_values
        $form_values['module_rule_start_date'] = $start_date;
        $form_values['module_rule_expire_date'] = $expire_date;

        if (count($this->context->controller->errors)) {
            return $this->context->controller->errors;
        }

        if (($id_addifyextendedhideprice = Tools::getValue('id_addifyextendedhideprice'))) {
            $call_back = 'update';
            $hideprice = new ExtHidePrice((int)$id_addifyextendedhideprice);
        } else {
            $hideprice = new ExtHidePrice();
        }

        $hideprice->title = $form_values['title'];
        $hideprice->priority = $form_values['priority'];
        $hideprice->active = $form_values['active'];

        $hideprice->hideprice = $form_values['hideprice'];

        $hideprice->hidecart = $form_values['hidecart'];
        $hideprice->active_custom_btn = $form_values['active_custom_btn'];
        $hideprice->custom_btn_title = $form_values['custom_btn_title'];
        $hideprice->custom_btn_link = $form_values['custom_btn_link'];
        $hideprice->custom_btn_text_color = $form_values['custom_btn_text_color'];
        $hideprice->custom_btn_background_color = $form_values['custom_btn_background_color'];

        $hideprice->rule_date_activate = $form_values['rule_date_activate'];
        $hideprice->module_rule_start_date= $form_values['module_rule_start_date'];
        $hideprice->module_rule_expire_date = $form_values['module_rule_expire_date'];
        
        $hideprice->groups_activate = $form_values['groups_activate'];
        $hideprice->groups = $form_values['groupBox'];
        
        $hideprice->product_type = $form_values['type'];
        $hideprice->selected_products_activate = $form_values['selected_products_activate'];
        $hideprice->product = $form_values['product'];

        $hideprice->category_activate = $form_values['category_activate'];
        $hideprice->categories = $form_values['category'];

        $hideprice->supplier_activate = $form_values['supplier_activate'];
        $hideprice->Supplier = $form_values['Supplier[]'];

        $hideprice->manufacturer_activate = $form_values['manufacturer_activate'];
        $hideprice->Manufacturer = $form_values['Manufacturer[]'];

        $hideprice->price_based_activate = $form_values['price_based_activate'];
        $hideprice->min_price = $form_values['min_price'];
        $hideprice->max_price = $form_values['max_price'];

        $hideprice->stock_based_activate = $form_values['stock_based_activate'];
        $hideprice->zero_stock_activate = $form_values['zero_stock_activate'];
        $hideprice->custom_stock_activate = $form_values['custom_stock_activate'];
        $hideprice->custom_stock_value = $form_values['custom_stock_value'];

        $hideprice->enabled_disabled_custom_message_mode = $form_values['enabled_disabled_custom_message_mode'];
        $hideprice->displaytext = $form_values['message'];

        if (!call_user_func(array($hideprice, $call_back))) {
            $this->context->controller->errors = sprintf($this->l('Something went wrong while performing operation %s'), $call_back);
        } else {
            
            $this->flushCache();

            if($call_back=='add') {
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=3&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name.'&active_tab=2');
            }
            else {
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=4&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name.'&active_tab=2');
            }
        }
    }
    public function flushCache()
    {
        Tools::clearSmartyCache();
        Tools::clearXMLCache();
        Media::clearCache();
        Tools::generateIndex();
        if (true === Tools::version_compare(_PS_VERSION_, '1.7.0.0', '>=') &&
            is_callable(array('Tools', 'clearAllCache'))) {
            Tools::clearAllCache();
        }
    }

    public function hookDisplayProductPriceBlock($param)
    {
        if (Configuration::get('addify_AS_live_mode') == 0 || $param['type'] != 'weight') {
            return;
        }
        
        $productId = '';
        $product = $param['product'];
        if (is_object($product)) {
            $catas = Product::getProductCategoriesFull($product->id);
            $productId = $product->id;
            $id_product_attribute = $product->id_product_attribute;
        } else {
            $catas = Product::getProductCategoriesFull($product['id_product']);
            $productId = $product['id_product'];
            $id_product_attribute = $product['id_product_attribute'];
        }

        $customerGroupid = (int)Group::getCurrent()->id;
        $id_supplier = (int)$product->id_supplier;
        $id_manufacturer = (int)$product->id_manufacturer;
        $stockQuantity = StockAvailable::getQuantityAvailableByProduct($productId);
        $ruleDat = ExtHidePrice::getRuleByProduct($productId, $catas, $customerGroupid, $id_supplier, $id_manufacturer, $stockQuantity);

        if ($ruleDat) {

            foreach ($ruleDat as $ruleData) {

                $text = ExtHidePrice::getTextDisplay((int)$ruleData['id_addifyextendedhideprice'], (int)$this->context->language->id);
                $hidePrice = $ruleData['hideprice'];

                $text_msg = '';
                $txtcolor = '';
                $textbg = '';

                if ($hidePrice == 1 && $ruleData['enabled_disabled_custom_message_mode'] == 1) {  
                    $text_msg = $text['displaytext'];
                }

                $hideCart = $ruleData['hidecart'];
                $active_custom_btn = '';
                $custom_btn_title = '';
                $custom_btn_link = '';
                $custom_btn_text_color = '';
                $custom_btn_background_color = '';

                if ($hideCart == 1) {  
                    $active_custom_btn = $ruleData['active_custom_btn'];
                    if ($active_custom_btn == 1) { 
                        $custom_btn_title = !empty($ruleData['custom_btn_title']) ? $ruleData['custom_btn_title'] : $this->l('Custom Button');
                        $custom_btn_link = !empty($ruleData['custom_btn_link']) ? $ruleData['custom_btn_link'] : '';
                        $custom_btn_text_color = !empty($ruleData['custom_btn_text_color']) ? $ruleData['custom_btn_text_color'] : 'white';
                        $custom_btn_background_color = !empty($ruleData['custom_btn_background_color']) ? $ruleData['custom_btn_background_color'] : 'black';
                    }
                }

                $priceHideCheck = ($hidePrice === 0) ? 'no' : 'yes';
                $hideCartCheck = ($hideCart === 0) ? 'no' : 'yes';

                $this->context->smarty->assign([
                    'ps_version' => _PS_VERSION_,
                    'hideCartCheck' => $hideCartCheck,
                    'priceHideCheck' => $priceHideCheck,
                    'productId' => $productId,
                    'text' => $text_msg,
                    'active_custom_btn' => $active_custom_btn,
                    'custom_btn_title' => $custom_btn_title,
                    'custom_btn_link' => $custom_btn_link,
                    'custom_btn_text_color' => $custom_btn_text_color,
                    'custom_btn_background_color' => $custom_btn_background_color,
                ]);

            return $this->display(__FILE__, 'views/templates/hook/extendhideprice.tpl');
            }
        }

         else {
            $this->context->smarty->assign([
                'ps_version' => _PS_VERSION_,
                'hideCartCheck' => 'no',
                'priceHideCheck' => 'no',
                'productId' => '',
                'text' => '',
                'active_custom_btn' => '',
                'custom_btn_title' => '',
                'custom_btn_link' => '',
                'custom_btn_text_color' => '',
                'custom_btn_background_color' => '',
            ]);

            return $this->display(__FILE__, 'views/templates/hook/extendhideprice.tpl'); // If rule not active
        }
    }


    public function hookDisplayHeader()
    {
        if (!$this->active) {
            return;
        }

        if (Configuration::get('addify_AS_live_mode') == 0) {
            return;
        } 

        if (true === Tools::version_compare(_PS_VERSION_, '1.7.0.0', '>=')) {
            $this->context->controller->addJS($this->_path . 'views/js/extaddifyhideprice17.js');
            $this->context->controller->addCSS($this->_path . 'views/css/extaddifyhideprice17.css');
        } 
        else {
            $this->context->controller->addJS($this->_path . 'views/js/extaddifyhideprice.js');
            $this->context->controller->addCSS($this->_path . 'views/css/extaddifyhideprice.css');
        }
    }
}
